/*
 * alarm-properties.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the UI for setting an alarm.
 */

const SOUND_FILES = "*.wav";

var rseRaces = ['elvaan.male', 'elvaan.female',
	'tarutaru.male', 'tarutaru.female',
	'mithra', 'galka', 'hume.male', 'hume.female' ];

var vanadielTimer;
var alarmManager;
var vanadielOffset = 0;

function init() {
    sizeToContent();
    var x = Math.floor((window.opener.outerWidth - window.outerWidth) / 2 + window.opener.screenX);
    var y = Math.floor((window.opener.outerHeight - window.outerHeight) / 2 + window.opener.screenY);
    window.moveTo(x, y);
    vanadielTimer = Components.classes["@xenoveritas.org/vanadiel-timer;1"]
			 .getService(Components.interfaces.nsIVanadielTimer);
    vanadielOffset = vanadielTimer.vanadielOffset;
    // grab the alarm manager
    alarmManager = Components.classes["@xenoveritas.org/vanadiel-alarm-manager;1"].
               getService(Components.interfaces.nsIVanadielAlarmManager);
    var label = document.getElementById("alarm.label");
    var aID = window.arguments.length >= 1 ? window.arguments[0] : null;
    if (aID != null || aID == "new") {
	document.getElementById("vanadielclock-set-alarm-window")
	    .setAttribute("title", document.getElementById("alarm-properties-strings").getString("window.title.create"));
    } else {
	// try and load an existing alarm from the alarm manager
	var alarm = alarmManager.getAlarmById(aID);
	label.value = alarm.name;
	if (alarm.sound == "none") {
	    document.getElementById("alarm-sound-none").selected = true;
	} else if (alarm.sound == "beep") {
	    document.getElementById("alarm-sound-beep").selected = true;
	} else {
	    document.getElementById("alarm-sound-custom").selected = true;
	    document.getElementById("alarm-sound-custom-box").value = alarm.sound;
	}
	// set up the existing data based on the alarm values
	if (alarm.type == "time") {
	    // figure out the day it sounds on
	    
	}
    }
    checkSoundBefore();
    checkAlarmSound();
}

function setDefaultName(name) {
    document.getElementById("alarm.label").defaultValue = name;
}

function checkSoundBefore() {
    document.getElementById("minutes.before").disabled = ! document.getElementById("sound-before-event").checked;
}

function checkAlarmSound() {
    var disabled = !document.getElementById("alarm-sound-custom").selected;
    document.getElementById("alarm-sound-custom-box").disabled = disabled;
    document.getElementById("alarm-sound-custom-browse").disabled = disabled;
    // there's really no point to having "test" enabled if "none" is chosen
    // Actually, there's little point in having it enabled if "beep" is chosen,
    // but I suppose people might want to know what "system default" is
    document.getElementById("alarm-sound-custom-test").disabled = document.getElementById("alarm-sound-none").selected;
}

function browseSound() {
    var strings = document.getElementById("alarm-properties-strings");
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
	.createInstance(nsIFilePicker);
    fp.appendFilter(strings.getString("sound.files") + " (" + SOUND_FILES + ")", SOUND_FILES);
    fp.init(window, strings.getString("sound.browse.title"), nsIFilePicker.modeOpen);
    var res = fp.show();
    if (res == nsIFilePicker.returnOK) {
	var soundFile = fp.file;
	document.getElementById("alarm-sound-custom-box").value = soundFile.path;
    }
}

function testSound() {
    // see what type of sound we're using
    if (document.getElementById("alarm-sound-none").selected) {
	// that was mighty pointless
	return;
    }
    var soundService = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    soundService.init();
    if (document.getElementById("alarm-sound-beep").selected) {
	soundService.beep();
    } else {
	soundService.playSystemSound(document.getElementById("alarm-sound-custom-box").value);
    }
}

/**
 * Creates a skeleton object for a given alarm.  This is used to "test"
 * the alarm using the "Test Alarm" button, or at least the secret keystroke
 * Accel-T
 */
function createAlarm() {
    var alarm = new Object();
    alarm.name = document.getElementById("alarm.label").value;
    alarm.sound = "none";
    if (document.getElementById("alarm-sound-beep").selected) {
	alarm.sound = "beep";
    } else {
	alarm.sound = document.getElementById("alarm-sound-custom-box").getAttribute("value");
    }
    var vanadielTime = vanadielTimer.time;
    var type = document.getElementById("type-tabbox").selectedTab.getAttribute("id");
    if (type == "time-tab") {
	// Create a new alarm based on the day.
	var dayName = document.getElementById("day-dropdown").selectedItem.getAttribute("element");
	var day = -1;
	// convert that day back to a number
	for (var d in vanaDays) {
	    if (dayName == vanaDays[d]) {
		day = d;
		break;
	    }
	}
	if (day < 0) {
	    alert("Unknown day " + dayName);
	    return;
	}
	alarm.smallIcon = "chrome://vanadielclock/skin/elements/" + dayName;
	alarm.icon = alarm.smallIcon + "32.png";
	alarm.smallIcon = alarm.smallIcon + ".png";
	alarm.description = document.getElementById("day-dropdown").selectedItem.label;
	alarm.time = vanadielTime - (vanadielTime % 691200);
	alarm.time += day * 86400;
	// if we're in the past, jump to the next week
	if (alarm.time < vanadielTime)
	    alarm.time += 691200;
	alarm.period = 691200;
	alarm.type = "time";
    } else if (type == "moon-tab") {
	// Create a new alarm based on the moon phase.
	var phase = document.getElementById("moon-dropdown").selectedItem.getAttribute("phase");
	alarm.type = "moon";
	alert("These aren't implemented yet.");
	return;
    } else if (type == "rse-tab") {
	alert("These aren't implemented yet.");
	return;
    } else if (type == "guild-tab") {
	alert("These aren't implemented yet.");
	return;
    } else {
	alert("You managed to choose an alarm type (" + type + ") that isn't listed.  Have a cookie!  But no alarm will be set, sorry.");
	return;
    }
    if (alarm.time < vanadielTime) {
	alert("Error: alarm set to " + alarm.time + " but current time is " + vanadielTime + ": assuming something is wonky and aborting.");
	return alarm;
    }
    dump("Creating an alarm at " + alarm.time + " (current time is " + vanadielTime + ")\n");
    return alarm;
}

function testAlarm() {
    var alarm = createAlarm();
    var alertService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
    alertService.showAlertNotification(alarm.icon, alarm.name, alarm.description, false, "don't care", null);
    testSound();
}

function acceptChanges() {
    // Create the alarm
    var jsAlarm = createAlarm();
    // create a new alarm
    var xpcAlarm = Components.classes["@xenoveritas.org/vanadiel-alarm;1"].
               createInstance(Components.interfaces.nsIVanadielAlarm);
    // set the properties
    xpcAlarm.name = jsAlarm.name;
    xpcAlarm.period = jsAlarm.period;
    xpcAlarm.description = jsAlarm.description;
    xpcAlarm.icon = jsAlarm.icon;
    xpcAlarm.smallIcon = jsAlarm.smallIcon;
    xpcAlarm.sound = jsAlarm.sound;
    xpcAlarm.type = jsAlarm.type;
    xpcAlarm.setAlarm(jsAlarm.time);
    alarmManager.addAlarm(xpcAlarm);
}
