/*
 * vanadiel-clock-panel.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Implements a Vana'diel clock in the browser window.
 *
 * Simple, minimally useful.
 */

var vanaClock = null;

function VanadielClock_load() {
    // prevent this from being called on every window
    if (document.getElementById("vanadiel-time") == null) {
	dump("Clock panel missing in " + window.location + "\n");
	return;
    }

    // init library
    VanadielClock_init();

    // Load preferences
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch);

    vanaClock = new VanadielClock();
    vanaClock.setVisible(prefs.getBoolPref("vanadiel.clock.show"));
    vanaClock.setShowMoonPhase(prefs.getBoolPref("vanadiel.clock.moon"));
    vanaClock.setShowDate(prefs.getBoolPref("vanadiel.clock.date"));
}

function VanadielClock_unload() {
    vanaClock.destroy();
    vanaClock = null;
}

/*
These are based on the old version and need to be redesigned.
The new version will listen for perference changes.
function startVanadielClock(showMoon, showDate) {
    // show the moon on null
    if (showMoon == null) {
	showMoon = true;
    }
    // don't show the date on null (kinda redundant, actually)
    if (showDate == null) {
	showDate = false;
    }
    if (vanadielClockInterval == null) {
	// immediately update the clock
	vanadielLastDay = -1;
	updateVanadielClock();
    } else {
	// this may have just toggled the moon/date
	vanadielMoonIcon = document.getElementById("vanadiel-moon");
	vanadielMoonIcon.style.display = showMoon ? "-moz-box" : "none";
	vanadielDateLabel = document.getElementById("vanadiel-date");
	vanadielDateLabel.style.display = showDate ? "-moz-box" : "none";
    }
}

function stopVanadielClock() {
    if (vanadielClockInterval != null) {
	vanadielClockPanel = document.getElementById("vanadiel-clock");
	vanadielClockPanel.style.display = "none";
	vanadielClockInterval = null;
    }
}*/

function VanadielClock() {
    // TODO: make the clock item an XBL element, and grab it based on an ID
    // parameter (so you can embed the clock anywhere)
    this.clockPanel = document.getElementById("vanadiel-clock");
    this.moonIcon = document.getElementById("vanadiel-moon");
    this.dateLabel = document.getElementById("vanadiel-date");
    this.dayIcon = document.getElementById("vanadiel-day");
    this.timeLabel = document.getElementById("vanadiel-time");
    // Grab the clock
    this.clock = VanadielClock_TimerService;
    if (!this.clock.active) {
	this.clock.start();
    }
    this.clock.addVanadielClockListener(this);
    this.vanadielTicked(this.clock.year, this.clock.month, this.clock.day, this.clock.weekday, this.clock.hour, this.clock.minute);
}

VanadielClock.prototype = {
    clock: null,
    clockPanel: null,
    moonIcon: null,
    dateLabel: null,
    dayIcon: null,
    timeLabel: null,
    lastMinute: -1,
    lastDay: -1,

    destroy: function() {
	this.clock.removeVanadielClockListener(this);
	this.clock = null;
    },

    setVisible: function(visible) {
	this.clockPanel.style.display = visible ? "-moz-box" : "none";
    },

    setShowMoonPhase: function(visible) {
	this.moonIcon.style.display = visible ? "-moz-box" : "none";
    },

    setShowDate: function(visible) {
	this.dateLabel.style.display = visible ? "-moz-box" : "none";
    },

    // Callback for the clock component
    vanadielTicked: function(year, month, day, weekday, hour, minute) {
	if (day != this.lastDay) {
	    this.lastDay = day;
	    // update date related stuff
	    var date = vanaLocalNames[weekday] + ", " + day + " / ";
	    date += month + " / ";
	    date += year;
	    var basicMoonPhase = this.clock.basicMoonPhase;
	    //dump("Calculated moon at " + this.clock.moonPercent + "% " + (this.clock.waxing ? "waxing" : "waning"));
	    //dump("\nPhase counter is " + this.clock.moonPhase + ", basic phase is " + basicMoonPhase + " (" + moonPhases[basicMoonPhase] + ")\n");
	    this.moonIcon.setAttribute("phase", moonPhases[basicMoonPhase]);
	    var moonPhaseLabel = this.clock.moonPercent + "% " + moonPhaseLocalNames[basicMoonPhase];
	    if (basicMoonPhase == 0 || basicMoonPhase == 6) {
		// add "waxing" or "waning" to this if you can't tell
		// (ie, when the moon is new or full, so you know if it's
		// moving to/from the "true" full/new state)
		if (this.clock.moonPhase != 0 && this.clock.moonPhase != 42) {
		    // But don't bother showing waxing/waning when the moon
		    // really is completely full or completely new - it should
		    // (hopefully) be obvious that it's going to move in the
		    // other direction.
		    moonPhaseLabel += " (" + (this.clock.waxing ? moonPhaseLocalWaxing : moonPhaseLocalWaning) + ")";
		}
	    }
	    this.moonIcon.setAttribute("tooltiptext", moonPhaseLabel);
	    this.clockPanel.setAttribute("tooltiptext", date);
	    this.dayIcon.setAttribute("tooltiptext", date);
	    this.dayIcon.setAttribute("day", vanaDays[weekday]);
	    this.dateLabel.setAttribute("value", date);
	    this.timeLabel.setAttribute("tooltiptext", date);
	}
	var label = hour + ":";
	if (minute < 10) {
	    label += "0";
	}
	label += minute;
	this.timeLabel.setAttribute("value", label);
    }
}
